#include "hmbdc/app/utils/Pingpong.hpp"
#include "hmbdc/app/netmap/NetContext.hpp"

#include <boost/program_options.hpp>

#include <tuple>

using namespace hmbdc::app;
using namespace hmbdc::app::netmap;
using namespace hmbdc::app::utils;
using namespace std;

int 
main(int argc, char** argv) {
    SingletonGuardian<SyncLogger> logGuard(std::cout);
    using namespace std;

    string type;
    string role;
    string netmapPort;
    uint16_t txRingIndex;
    vector<uint16_t> cpus;
    size_t skipFirst;
    uint16_t msgSize;
    uint16_t msgPerSec;
    uint32_t runTime;

    auto CONFIG = R"|(
{
    "nmResetWaitSec" : 5,
    "ping" : {
        "topicRegex" : "ping"
    },
    "pong" : {
        "topicRegex" : "pong"
    }
}
)|";

    namespace po = boost::program_options;
    po::options_description desc("Allowed options");
    auto helpStr = 
    "Warning: hmbdc netmap network messaging uses multicast functions and it is imperative that multicast is enabled (stop the firewall?) on your network for this to work."
    "This program can be used to test round trip hmbdc netmap message latency among a group of hosts."
    "The source code is in example dir: ping-pong-netmap.cpp"
    "\nUsage example: \nponger$ sudo ./ping-pong-netmap -r pong -I netmap:p3p2"
    "\npinger$ sudo ./ping-pong-netmap -r ping -I netmap:eno1"
    "\nwould start the test using two host ponger and pinger with default settings.";
    ;
    desc.add_options()
    ("help", helpStr)
    ("role,r", po::value<string>(&role)->default_value("pong"), "ping or pong")
    ("netmapPort,I", po::value<string>(&netmapPort), "netmap port to use, for example: netmap:eth0")
    ("txRingIndex", po::value<uint16_t>(&txRingIndex)->default_value(0), "a netmap dev could contain multiple tx rings - specify which one to use in sending engine")
    ("msgSize", po::value<uint16_t>(&msgSize)->default_value(16), "msg size: 16-1000")
    ("msgPerSec", po::value<uint16_t>(&msgPerSec)->default_value(1), "msg per second up to 65535")
    ("cpus", po::value(&cpus)->multitoken()->default_value({0, 1, 2}, "0 1 2"), "specify the 3 cpu index numbers used in the test (recv, send engines, and 1 for msg client)")
    ("skipFirst", po::value<size_t>(&skipFirst)->default_value(1u), "skipp the first N results (buffering & warming up stage) when collecting latency stats")
    ("runTime", po::value<uint32_t>(&runTime)->default_value(0), "how many seconds does the test last before exit. By default it runs forever")
;

    po::positional_options_description p;
    po::variables_map vm;
    po::store(po::command_line_parser(argc, argv).
        options(desc).positional(p).run(), vm);
    po::notify(vm);


    if (vm.count("help")) {
        cout << desc << "\n";
        return 0;
    }
    Config config(CONFIG);
    config.put("skipFirst", skipFirst);
    config.put("msgSize", msgSize);
    config.put("msgPerSec", msgPerSec);    
    config.put("runTime", runTime);

    if (role == "ping" ) {
        config.put("ping", true);
        config.put("ping.netmapPort", netmapPort + "-" + to_string(txRingIndex));
        config.put("pong.netmapPort", netmapPort);
    	return pingpong<NetContext>(config, cpus);
    } else if (role == "pong") {
        config.put("ping", false);
        config.put("pong.netmapPort", netmapPort + "-" + to_string(txRingIndex));
        config.put("ping.netmapPort", netmapPort);
    	return pingpong<NetContext>(config, cpus);
    } else {
        cout << desc << "\n";
        return -1;
    }
}

