#pragma once 

#include <type_traits>
#include <tuple>

namespace hmbdc {
template <typename T>
struct function_traits: function_traits<decltype(&T::operator())>
{};

template <typename ClassType, typename ReturnType, typename... Args>
struct function_traits<ReturnType(ClassType::*)(Args...) const> {
    enum { arity = sizeof...(Args) };
    typedef ReturnType result_type;

    template <size_t i>
    struct arg {
        using type = typename std::tuple_element<i, std::tuple<Args...>>::type;
    };
};

template <typename ClassType, typename ReturnType, typename... Args>
struct function_traits<ReturnType(ClassType::*)(Args...)> {
    enum { arity = sizeof...(Args) };

    typedef ReturnType result_type;

    template <size_t i>
    struct arg {
        using type = typename std::tuple_element<i, std::tuple<Args...>>::type;
    };
};

template <typename Base, typename Derived>
struct is_first_base_of{
    enum {
        value = std::is_base_of<Base, Derived>::value 
               //&& (Base const*)((Derived const*)0xffff) == ((Base const*)0xffff) - not supported on some compilers
    };
};

template <typename T, typename Tuple>
struct index_in_tuple;

template <typename T, typename... Types>
struct index_in_tuple<T, std::tuple<T, Types...>> {
    static const std::size_t value = 0;
};

template <typename T>
struct index_in_tuple<T, std::tuple<>> {
    static const std::size_t value = 0;
};

template <typename T, typename U, typename... Types>
struct index_in_tuple<T, std::tuple<U, Types...>> {
    static const std::size_t value = 1 + index_in_tuple<T, std::tuple<Types...>>::value;
};


template <typename T, typename Tuple>
struct add_if_not_in_tuple {
    using type = std::tuple<T>;
};

template <typename T, typename... Types>
struct add_if_not_in_tuple<T, std::tuple<Types ...>> {
private:
    using Tuple = std::tuple<Types ...>;
public:    
    using type = typename std::conditional<
        index_in_tuple<T, Tuple>::value == sizeof...(Types)
        , std::tuple<Types ..., T>
        , Tuple
        >::type;
};

template <typename Tuple1, typename Tuple2>
struct merge_tuple_unique;

template <typename Tuple1>
struct merge_tuple_unique<Tuple1, std::tuple<>> {
    using type = Tuple1;
};

template <typename Tuple1, typename T, typename... Types>
struct merge_tuple_unique<Tuple1, std::tuple<T, Types...>> {
private:
    using step1 = typename add_if_not_in_tuple<T, Tuple1>::type;
public:
    using type = typename merge_tuple_unique<step1, std::tuple<Types...>>::type;
};

template <typename Tuple>
struct max_size_in_tuple {
    static const std::size_t value = 0;
};

template <typename T, typename... Ts>
struct max_size_in_tuple<std::tuple<T, Ts...>> {
private:
    static const std::size_t step1 = max_size_in_tuple<std::tuple<Ts...>>::value;
public:
    static const std::size_t value = step1 > sizeof(T) ? step1 : sizeof(T);
};

// template <typename... Types> struct concat_tuple;
// template <template <typename...> typename P, typename... Ts, typename... Us>
// struct concat_tuple<P<Ts...>, P<Us...>> {
//     using type = P<Ts..., Us...>;
// };

// template <typename Pack1, typename Pack2, typename... Types>
// struct concat_tuple<Pack1, Pack2, Types...> : concat_tupl<Pack1, typename concat_tupl<Pack2, Types...>::type> {};

}
