#include "hmbdc/Copyright.hpp"
#pragma once

#include "hmbdc/app/utils/EpollTask.hpp"
#include "hmbdc/app/Config.hpp"
#include "hmbdc/text/StringTrieSet.hpp"
#include "hmbdc/comm/inet/Misc.hpp"
#include "hmbdc/time/Timers.hpp"
#include "hmbdc/Exception.hpp"

#include <memory>
#include <string>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <arpa/inet.h>

namespace hmbdc { namespace app { namespace udpcast {

struct EpollFd : utils::EpollFd {
    EpollFd(Config const& cfg) {
        using namespace std;
        fd = socket(AF_INET, SOCK_DGRAM, 0);
        if(fd < 0) {
            HMBDC_THROW(runtime_error, "failed to create socket");
        }
        auto iface = 
            comm::inet::getLocalIpMatchMask(cfg.getExt<string>("ifaceAddr")); 
        struct in_addr localInterface;
        localInterface.s_addr = inet_addr(iface.c_str());
        if (setsockopt(fd, IPPROTO_IP, IP_MULTICAST_IF
            , (char *)&localInterface, sizeof(localInterface)) < 0) {
            HMBDC_THROW(runtime_error, "failed to set ifaceAddr " << cfg.getExt<string>("ifaceAddr"));
        }
    }
};

struct Transport : EpollFd {
    using ptr = std::shared_ptr<Transport>;

    Transport(Config const& cfg)
    : EpollFd(cfg)
    , config_(cfg)
    , mtu_(config_.getExt<size_t>("mtu")) {
        mtu_ -= (8u + 20u);  // 8bytes udp header and 20bytes ip header
        cfg (hmbdcName_, "hmbdcName")
            (schedPolicy_, "schedPolicy")
            (schedPriority_, "schedPriority")
        ;
    }

    char const* hmbdcName() const { 
        return this->hmbdcName_.c_str();
    }

    std::tuple<char const*, int> schedSpec() const {
        return std::make_tuple(this->schedPolicy_.c_str(), this->schedPriority_);
    }

    bool operator == (Transport const& other ) const {
        return &config_ == &other.config_;
    }

    bool operator < (Transport const& other ) const {
        return &config_ < &other.config_;
    }

protected:
    std::string hmbdcName_;
    std::string schedPolicy_;
    int schedPriority_;
    Config const config_;
    size_t mtu_;
};

}}}
