#include "hmbdc/Copyright.hpp"
#pragma once
#include "hmbdc/pattern/GuardedSingleton.hpp"
#include "hmbdc/Config.hpp"
#include "hmbdc/Compile.hpp"
#include <memory>       
#include <unistd.h>

namespace hmbdc { namespace app { namespace utils {

struct EpollFd;
struct EpollTask 
: pattern::GuardedSingleton<EpollTask> {
	void add(uint32_t events, EpollFd&);
    bool del(EpollFd&);
    void poll();

private:
	friend pattern::SingletonGuardian<EpollTask>;
	EpollTask(size_t maxFdCount = HMBDC_EPOLL_FD_MAX);

	~EpollTask();

    bool stopped_;
	int epollFd_;
	// std::thread pollThread_;
};

extern std::unique_ptr<hmbdc::pattern::SingletonGuardian<hmbdc::app::utils::EpollTask>> gEpollTaskGuard;

struct EpollFd {
    EpollFd(EpollFd const&) = delete;
    EpollFd& operator = (EpollFd const&) = delete;
	EpollFd()
    : fd(-1)
    , fdReady_(false)
    , fdReadyLocal_(false) 
    {}

    virtual 
    ~EpollFd() {
        if (fd > 0) {
            utils::EpollTask::instance().del(*this);
            close(fd);
        }
    }

    bool isFdReady() {
        if (hmbdc_likely(fdReadyLocal_)) {
            return true;
        } else {
            // return fdReadyLocal_ = __sync_val_compare_and_swap(&fdReady_, true, false);
            return fdReadyLocal_ = __atomic_exchange_n(&fdReady_, false, __ATOMIC_RELAXED);
        }
    }

    bool checkErr() {
        fdReadyLocal_ = false;
        if (errno != EAGAIN) {
            if (!utils::EpollTask::instance().del(*this)) {
            } else {
                close(fd);
                fd = -1;
            }
            return false;
        }
        return true;
    }

    int fd;
    
private:
	friend struct EpollTask;
    bool fdReady_;
    bool fdReadyLocal_;
};
}}}

