#include "hmbdc/Copyright.hpp"
#pragma once
#include "hmbdc/app/utils/EpollTask.hpp"
#include "hmbdc/app/Base.hpp"

#include <utility>

namespace hmbdc { namespace app { namespace utils {

struct NetContextUtil {
protected:
    template <typename CcNetContext, typename RunnerContext, typename CcContext>
	static
    typename std::enable_if<RunnerContext::cpa::has_pool, void>::type
    createMinimumNetContext(CcNetContext& netCtx
        , RunnerContext& runnerCtx
        , CcContext& ctx
    	, Config::Base const* cfgIn
    	, char const* sendSec
    	, char const* recvSec
    	, size_t maxMessageSize
        , uint8_t runningUsingThreadIndex
        , char const* listenToTopic) {

        runnerCtx.start(1, 1u << runningUsingThreadIndex);
    	if (sendSec == nullptr || sendSec[0]) {
    		auto eng = netCtx.createSendTransportEngine(
    			Config(*cfgIn, sendSec)
    			, maxMessageSize?maxMessageSize:ctx.maxMessageSize()
    		);
    		runnerCtx.addToPool(*eng);
    	}
    	if (recvSec == nullptr || recvSec[0]) {
    		auto eng = netCtx.createRecvTransportEngine(
    			Config(*cfgIn, recvSec)
    			, ctx.buffer()
    		);
    		runnerCtx.addToPool(*eng);
            netCtx.listenTo(Topic(listenToTopic));
    	}
    }

    void checkEpollTaskInitialization() {
        auto& pg = utils::gEpollTaskGuard;
        if (!pg) {
            pg.reset(new SingletonGuardian<utils::EpollTask>);
        }
    }
};
}}}
