#include "hmbdc/Copyright.hpp"
#pragma once

#include "hmbdc/pattern/LockFreeBufferMisc.hpp"
#include "hmbdc/os/Allocators.hpp"
#include "hmbdc/Config.hpp"

#include <utility>
#include <functional>
#include <limits>
#include <stdexcept>
#include <cstddef>

namespace hmbdc { namespace pattern {
    struct MonoLockFreeBuffer {
    using Sequence = HMBDC_SEQ_TYPE;
    using iterator = hmbdc::pattern::lf_misc::iterator<Sequence>;
    using value_type = void *;
    using DeadConsumer = hmbdc::pattern::lf_misc::DeadConsumer;
    enum {
        max_parallel_consumer = 0xffff
    };

    template <typename Allocator = os::DefaultAllocator>
    MonoLockFreeBuffer(size_t, uint32_t, Allocator& allocator = os::DefaultAllocator::instance);
    ~MonoLockFreeBuffer();
    
    size_t maxItemSize() const;
    size_t capacity() const;
    void put(void const*, size_t sizeHint = 0);  
    template <typename T> void put(T const& item) {put(&item, sizeof(item));}
    template <typename T> void putSome(T const& item) {put(&item, std::min(sizeof(item), maxItemSize()));}
    template <typename T, typename ...Args>
    void putInPlace(Args&&... args) {
        auto s = claim();
        new (*s) T(std::forward<Args>(args)...);
        commit(s);
    }
    template <typename T, typename ...Args>
    bool tryPutInPlace(Args&&... args) {
        auto s = tryClaim();
        if (!s) return false;
        new (*s) T(std::forward<Args>(args)...);
        commit(s);
        return true;
    }
    
    bool tryPut(void const*, size_t sizeHint = 0);
    bool isFull() const;
    Sequence readSeq() const;
    iterator claim();
    iterator tryClaim();
    iterator claim(size_t);
    iterator tryClaim(size_t);
    iterator killClaim();
    iterator killClaim(size_t);
    void commit(iterator);
    void commit(iterator, size_t);
    void take(void *, size_t = 0);
    bool tryTake(void *, size_t = 0);
    iterator peek();
    size_t peek(iterator&, iterator&
        , size_t maxPeekSize = std::numeric_limits<size_t>::max());
    /**
     * @brief if size not matching - please refer to the impl for details
     */
    void wasteAfterPeek(iterator, size_t, bool = false);
    size_t remainingSize() const;
    size_t parallelConsumerAlive() const {return 1;}
    void reset();

    static 
    size_t footprint(size_t, uint32_t);
private:
    std::ptrdiff_t impl_;
    std::function<void()> freer_;
};
}}
