#include "hmbdc/Copyright.hpp"
#pragma once

#include "hmbdc/pattern/LockFreeBufferMisc.hpp"
#include "hmbdc/time/Timers.hpp"
#include "hmbdc/Exception.hpp"
#include "hmbdc/Config.hpp"

#include <stdexcept>

namespace hmbdc { namespace pattern {

struct PoolConsumer {
    friend struct PoolImpl;

    template <typename Buffer>
    friend struct PoolTImpl;
    friend struct PoolMinusImpl;
    explicit PoolConsumer(bool interestedInMessages, time::TimerManager* tmPtr);
    virtual ~PoolConsumer();
    void stopped(std::exception const&) noexcept;
    bool dropped() noexcept;
    void messageDispatchingStarted(uint16_t threadId) {
        if (hmbdc_unlikely(!messageDispatchingStarted_)) {
            messageDispatchingStartedCb(threadId);
            messageDispatchingStarted_ = true;
        }
    }
    void invoked(uint16_t threadId);

protected:
    using BufIt = lf_misc::iterator<HMBDC_SEQ_TYPE>;

private:
    void reset();
    bool handleRange(BufIt begin, 
        BufIt end, uint16_t threadId) noexcept;
    bool handleInvokeOnly(uint16_t threadId) noexcept;
    virtual void messageDispatchingStartedCb(uint16_t threadId) {}
    virtual void handleRangeImpl(BufIt begin, 
        BufIt end, uint16_t threadId){};
    virtual void invokedCb(uint16_t threadId) {}
    virtual void stoppedCb(std::exception const&) {}
    virtual bool droppedCb() { return true; }
    
    hmbdc::time::TimerManager* tmPtr;
    uint64_t poolThreadAffinity;
    uint16_t droppedCount;
    uint64_t skippedPoolThreadMask_;
    HMBDC_SEQ_TYPE nextSeq_ __attribute__((__aligned__(SMP_CACHE_BYTES)));
    bool interestedInMessages;
    bool messageDispatchingStarted_;
};

}} // end namespace hmbdc::pattern
