#include "hmbdc/Copyright.hpp"
#pragma once

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/trie_policy.hpp>
#include <string>
#include <unistd.h>
#include <string.h>


namespace hmbdc { namespace text {


namespace stringtrieset_detail {
using namespace std;

struct stringAccessTraits {
	using size_type = size_t;
	using key_type = std::string;
    using __rebind_k = std::allocator<char>::template rebind<key_type>;
    using key_const_reference = typename __rebind_k::other::const_reference;
    enum { reverse = false };
    using const_iterator = const char*;
    using e_type = const char;

    enum {
        min_e_val = __gnu_pbds::detail::__numeric_traits<char>::__min,
        max_e_val = __gnu_pbds::detail::__numeric_traits<char>::__max,
        max_size = max_e_val - min_e_val + 1
    };

    /// Returns a const_iterator to the first element of
    /// key_const_reference agumnet.
    inline static const_iterator
    begin(key_const_reference ck) { return ck.c_str(); }

    /// Returns a const_iterator to the after-last element of
    /// key_const_reference argument.
    inline static const_iterator
    end(key_const_reference ck) { return ck.c_str() + ck.size(); }

    /// Maps an element to a position.
    inline static size_type
    e_pos(e_type e) { return static_cast<size_type>(e - min_e_val); }

};

template<typename Node_CItr,
    typename Node_Itr,
    typename _ATraits,
    typename _Alloc
>
struct 
string_wildcard_trie_search
: __gnu_pbds::detail::trie_policy_base<Node_CItr,Node_Itr,_ATraits, _Alloc> {
private:
    using base_type = __gnu_pbds::detail::trie_policy_base<Node_CItr, Node_Itr, _ATraits, _Alloc>;
    using node_iterator = typename base_type::node_iterator;
    using node_const_iterator = typename base_type::node_const_iterator;
    using access_traits = typename base_type::access_traits;
    using size_type = typename base_type::size_type;

    bool
    search_recursive(Node_Itr nd_it, Node_Itr end_nd_it
        , typename access_traits::const_iterator b
        , typename access_traits::const_iterator e
        , size_t len) {
        if (nd_it == end_nd_it) {
            return false;
        }
        auto r = nd_it.valid_prefix();
        auto pref_b = r.first + len;
        auto pref_e = r.second;

        while (pref_b != pref_e && b != e) {
            if (*pref_b != *b) {
                return false;
            }
            pref_b++;
            b++;
        }
        if (pref_b != pref_e) { //already seeing mismatching
            return false;
        }

        if (b == e && nd_it.m_p_nd->m_type == 1) { //perfect match a leaf?
            return true;
        }

        if (b != e && nd_it.m_p_nd->m_type == 1 && (*nd_it)->second) { //wildcard match?
            return true;
        }

        const size_type num_children = nd_it.num_children();
        len = pref_b - r.first;
        for (size_type i = 0; i < num_children; ++i) {
            if (search_recursive(nd_it.get_child(i), end_nd_it, b, e, len)) {
                return true;
            }
        }
        return false;
    }

public:
    bool 
    search_wildcardly(typename access_traits::const_iterator b
        , typename access_traits::const_iterator e) {
        Node_Itr nd_it = this->node_begin();
        Node_Itr end_nd_it = this->node_end();
        return search_recursive(nd_it, end_nd_it, b, e, 0);
    }
    void operator()(node_iterator /*nd_it*/, node_const_iterator /*end_nd_it*/) const{}
};

struct StringTrieSet
: private __gnu_pbds::trie<std::string, bool, stringAccessTraits, __gnu_pbds::pat_trie_tag, 
         string_wildcard_trie_search, std::allocator<char>>  {
    using Base = __gnu_pbds::trie<std::string, bool, stringAccessTraits, __gnu_pbds::pat_trie_tag, 
         string_wildcard_trie_search, std::allocator<char>>;
    
    using Base::begin;
    using Base::end;
    using Base::size;

    void add(std::string const& s) {
        if (s[s.size() - 1] == '*') {
    	   (*this)[s.substr(0, s.size() - 1)] = true;
        } else {
            (*this)[s];
        }
    }

    void erase(std::string const& s) {
        if (s[s.size() - 1] == '*') {
           Base::erase(s.substr(0, s.size() - 1));
        } else {
           Base::erase(s);
        }
    } 	

    bool check(char const*b, size_t len) const {
        return const_cast<StringTrieSet*>(this)->search_wildcardly(b, b + len);
    }       

    bool check(std::pair<char const*, char const*> const& str) const {
        return const_cast<StringTrieSet*>(this)->search_wildcardly(str.first, str.second);
    }       

};

} //stringtrieset_detail 
}}




