#include "hmbdc/Copyright.hpp"
#pragma once
#include "hmbdc/Exception.hpp"
// #include "hmbdc/pattern/Typed.h"
#include <string.h>
#include <string>
#include <stdexcept>
#include <functional>
#include <algorithm>
    

namespace hmbdc { namespace text { 
struct TypedStringHash;
template <char const NAME[], uint16_t SIZE>
struct TypedString
{
private:
    using ThisT = TypedString<NAME, SIZE>;
    char v_[SIZE]; 
    friend struct std::hash<TypedString<NAME, SIZE>>;

public:
    using rawType = char[SIZE];
    enum{capacity = SIZE,};

    TypedString(){v_[0] = 0;}
    explicit TypedString(char const* s, size_t len = SIZE) {strncpy(v_, s, SIZE);}
    explicit TypedString(std::string const& s) {strncpy(v_, s.c_str(), SIZE);}
    static char const* typeName() { return NAME; }
    char const* c_str() const { return v_; }
    bool operator == (ThisT const& other) const {
        return strncmp(v_, other.v_, SIZE) == 0;
    }
    bool operator != (ThisT const& other) const {
        return strncmp(v_, other.v_, SIZE) != 0;
    }
    bool operator < (ThisT const& other) const {
        return strncmp(v_, other.v_, SIZE) < 0;
    }
    friend
    std::ostream& operator << (std::ostream& os, ThisT const& s) {
        char tmp[SIZE + 1];
        strncpy(tmp, s.v_, SIZE);
        tmp[SIZE] = 0;
        os << tmp;
        return os;
    }
    friend
    std::istream& operator >> (std::istream& is, ThisT& s) {
        std::string tmp;
        is >> tmp;
        strncpy(s.v_, tmp.c_str(), SIZE);
        return is;
    }

    void clear() { v_[0] = 0; }
    size_t size() const {
        char const* b = v_;
        return std::find(b, b + SIZE, '\x00') - b;
    }

    size_t copyTo(char* to) const {
        char const* b = v_;
        char * p = to;
        for (; p != to + capacity && *b;) {
            *(p++) = *(b++);
        }
        return p - to;
    }
};

}}

namespace std {
    template <char const NAME[], uint16_t SIZE>
    struct hash<hmbdc::text::TypedString<NAME, SIZE>> {
        size_t operator()(const hmbdc::text::TypedString<NAME, SIZE>& x) const {
            uint32_t hash, i;
            for(hash = i = 0; i < SIZE && x.v_[i]; ++i)
            {
                hash += x.v_[i];
                hash += (hash << 10);
                hash ^= (hash >> 6);
            }
            hash += (hash << 3);
            hash ^= (hash >> 11);
            hash += (hash << 15);
            return hash;
        }
    };
};
